@@ -6,10 +6,11 @@ import healpy as hp
 import numpy as np
 from pixell import colorize, enmap, enplot
 from pspy import pspy_utils, so_mcm, sph_tools
-from pspy.cov_fortran import cov_fortran
+from pspy.cov_fortran.cov_fortran import cov_compute as cov_fortran
+from pspy.mcm_fortran.mcm_fortran import mcm_compute as mcm_fortran
 
 
-def cov_coupling_spin0(win, lmax, niter=3, save_file=None, threshold=None):
+def cov_coupling_spin0(win, lmax, niter=3, save_file=None, l_exact=None, l_band=None, l_toep=None):
     """compute the coupling kernels corresponding to the T only covariance matrix
         
    Parameters
@@ -24,13 +25,14 @@ def cov_coupling_spin0(win, lmax, niter=3, save_file=None, threshold=None):
       the number of iteration performed while computing the alm
     save_file: string
       the name of the file in which the coupling kernel will be saved (npy format)
-    threshold: integer
-      for approximating the cov computation, threshold is the max |l1-l2| consider
-      in the calculation
 
     """
 
     coupling_dict = {}
+    if l_toep is None: l_toep = lmax
+    if l_band is None: l_band = lmax
+    if l_exact is None: l_exact = lmax
+
     if type(win) is not dict:
         sq_win = win.copy()
         sq_win.data *= sq_win.data
@@ -38,19 +40,29 @@ def cov_coupling_spin0(win, lmax, niter=3, save_file=None, threshold=None):
         wcl = hp.alm2cl(alm)
         l = np.arange(len(wcl))
         wcl *= (2 * l + 1) / (4 * np.pi)
-        coupling = np.zeros((1, lmax, lmax))
-        if threshold is None:
-            cov_fortran.calc_cov_spin0_single_win(wcl, coupling.T)
-        else:
-            cov_fortran.calc_cov_spin0_single_win_threshold(wcl, threshold, coupling.T)
+        coupling = np.zeros((lmax, lmax))
+        
 
-            
-        coupling_dict["TaTcTbTd"] = coupling[0] + coupling[0].T - np.diag(np.diag(coupling[0])) #coupling[0]
-        coupling_dict["TaTdTbTc"] = coupling[0] + coupling[0].T - np.diag(np.diag(coupling[0])) #coupling[0]
+        mcm_fortran.calc_coupling_spin0(wcl,
+                                       l_exact,
+                                       l_band,
+                                       l_toep,
+                                       coupling.T)
+
+        
+        if l_toep < lmax:
+            # For toepliz fill the matrix
+            coupling = so_mcm.format_toepliz_fortran(coupling, l_toep, lmax)
+
+        mcm_fortran.fill_upper(coupling.T)
+
+        coupling_dict["TaTcTbTd"] = coupling
+        coupling_dict["TaTdTbTc"] = coupling
+        
     else:
         wcl = {}
-        for s in ["TaTcTbTd","TaTdTbTc"]:
-            n0, n1, n2, n3 = [s[i, i + 2] for i in range(4)]
+        for s in ["TaTcTbTd", "TaTdTbTc"]:
+            n0, n1, n2, n3 = [s[i * 2:(i + 1) * 2] for i in range(4)]
             sq_win_n0n1 = win[n0].copy()
             sq_win_n0n1.data *= win[n1].data
             sq_win_n2n3 = win[n2].copy()
@@ -62,17 +74,24 @@ def cov_coupling_spin0(win, lmax, niter=3, save_file=None, threshold=None):
             wcl[n0+n1+n2+n3] *= (2 * l + 1) / (4 * np.pi)
 
         coupling = np.zeros((2, lmax, lmax))
-        cov_fortran.calc_cov_spin0(wcl["TaTcTbTd"], wcl["TaTdTbTc"], coupling.T)
-        coupling_dict["TaTcTbTd"] = coupling[0] + coupling[0].T - np.diag(np.diag(coupling[0]))
-        coupling_dict["TaTdTbTc"] = coupling[1] + coupling[1].T - np.diag(np.diag(coupling[1]))
+        
+        cov_fortran.calc_cov_spin0(wcl["TaTcTbTd"], wcl["TaTdTbTc"], l_exact, l_band, l_toep,  coupling.T)
+        
+        for id_cov, name in enumerate(["TaTcTbTd", "TaTdTbTc"]):
+            if l_toep < lmax:
+                coupling[id_cov] = so_mcm.format_toepliz_fortran(coupling[id_cov], l_toep, lmax)
+            mcm_fortran.fill_upper(coupling[id_cov].T)
 
+            coupling_dict[name] = coupling[id_cov]
+
+        
     if save_file is not None:
         np.save("%s.npy"%save_file, coupling)
 
     return coupling_dict
 
 
-def cov_coupling_spin0and2_simple(win, lmax, niter=3, save_file=None, planck=False):
+def cov_coupling_spin0and2_simple(win, lmax, niter=3, save_file=None, planck=False, l_exact=None, l_band=None, l_toep=None):
     """Compute the coupling kernels corresponding to the T and E covariance matrix
 
     Parameters
@@ -100,6 +119,11 @@ def cov_coupling_spin0and2_simple(win, lmax, niter=3, save_file=None, planck=Fal
                 "PaTcPbPd", "PaPdPbTc", "PaPcPbTd", "PaTdPbPc"]
                 
     coupling_dict = {}
+    
+    if l_toep is None: l_toep = lmax
+    if l_band is None: l_band = lmax
+    if l_exact is None: l_exact = lmax
+
     if type(win) is not dict:
         sq_win = win.copy()
         sq_win.data *= sq_win.data
@@ -108,7 +132,18 @@ def cov_coupling_spin0and2_simple(win, lmax, niter=3, save_file=None, planck=Fal
         l = np.arange(len(wcl))
         wcl *= (2 * l + 1) / (4 * np.pi)
         coupling = np.zeros((1, lmax, lmax))
-        cov_fortran.calc_cov_spin0and2_single_win_simple(wcl, coupling.T)
+    
+
+        mcm_fortran.calc_coupling_spin0(wcl,
+                                       l_exact,
+                                       l_band,
+                                       l_toep,
+                                       coupling[0].T)
+
+        
+        if l_toep < lmax:
+            # For toepliz fill the matrix
+            coupling[0] = so_mcm.format_toepliz_fortran(coupling[0], l_toep, maxl)
 
         indexlist=np.zeros(32, dtype=np.int8)
 
@@ -116,6 +151,7 @@ def cov_coupling_spin0and2_simple(win, lmax, niter=3, save_file=None, planck=Fal
             coupling_dict[name] = coupling[index] + coupling[index].T - np.diag(np.diag(coupling[index]))
     else:
         wcl={}
+        
         for s in win_list:
 
             n0, n1, n2, n3 = [s[i * 2:(i + 1) * 2] for i in range(4)]
@@ -135,30 +171,28 @@ def cov_coupling_spin0and2_simple(win, lmax, niter=3, save_file=None, planck=Fal
         coupling = np.zeros((32, lmax, lmax))
 
         if planck==True:
-            cov_fortran.calc_cov_spin0and2_simple_planck(wcl["TaTcTbTd"], wcl["TaTdTbTc"], wcl["PaPcPbPd"], wcl["PaPdPbPc"],
-                                                  wcl["TaTcPbPd"], wcl["TaPdPbTc"], wcl["PaPcTbTd"], wcl["PaTdTbPc"],
-                                                  wcl["TaPcTbPd"], wcl["TaPdTbPc"], wcl["TaTcTbPd"], wcl["TaPdTbTc"],
-                                                  wcl["TaPcTbTd"], wcl["TaTdTbPc"], wcl["TaPcPbTd"], wcl["TaTdPbPc"],
-                                                  wcl["TaPcPbPd"], wcl["TaPdPbPc"], wcl["PaPcTbPd"], wcl["PaPdTbPc"],
-                                                  wcl["TaTcPbTd"], wcl["TaTdPbTc"], wcl["PaTcTbTd"], wcl["PaTdTbTc"],
-                                                  wcl["PaTcPbTd"], wcl["PaTdPbTc"], wcl["PaTcTbPd"], wcl["PaPdTbTc"],
-                                                  wcl["PaTcPbPd"], wcl["PaPdPbTc"], wcl["PaPcPbTd"], wcl["PaTdPbPc"],
-                                                  coupling.T)
-
+            doPlanck = 1
         else:
-            cov_fortran.calc_cov_spin0and2_simple(wcl["TaTcTbTd"], wcl["TaTdTbTc"], wcl["PaPcPbPd"], wcl["PaPdPbPc"],
-                                                  wcl["TaTcPbPd"], wcl["TaPdPbTc"], wcl["PaPcTbTd"], wcl["PaTdTbPc"],
-                                                  wcl["TaPcTbPd"], wcl["TaPdTbPc"], wcl["TaTcTbPd"], wcl["TaPdTbTc"],
-                                                  wcl["TaPcTbTd"], wcl["TaTdTbPc"], wcl["TaPcPbTd"], wcl["TaTdPbPc"],
-                                                  wcl["TaPcPbPd"], wcl["TaPdPbPc"], wcl["PaPcTbPd"], wcl["PaPdTbPc"],
-                                                  wcl["TaTcPbTd"], wcl["TaTdPbTc"], wcl["PaTcTbTd"], wcl["PaTdTbTc"],
-                                                  wcl["PaTcPbTd"], wcl["PaTdPbTc"], wcl["PaTcTbPd"], wcl["PaPdTbTc"],
-                                                  wcl["PaTcPbPd"], wcl["PaPdPbTc"], wcl["PaPcPbTd"], wcl["PaTdPbPc"],
-                                                  coupling.T)
+            doPlanck = 0
+            
+            
+        cov_fortran.calc_cov_spin0and2_simple(wcl["TaTcTbTd"], wcl["TaTdTbTc"], wcl["PaPcPbPd"], wcl["PaPdPbPc"],
+                                              wcl["TaTcPbPd"], wcl["TaPdPbTc"], wcl["PaPcTbTd"], wcl["PaTdTbPc"],
+                                              wcl["TaPcTbPd"], wcl["TaPdTbPc"], wcl["TaTcTbPd"], wcl["TaPdTbTc"],
+                                              wcl["TaPcTbTd"], wcl["TaTdTbPc"], wcl["TaPcPbTd"], wcl["TaTdPbPc"],
+                                              wcl["TaPcPbPd"], wcl["TaPdPbPc"], wcl["PaPcTbPd"], wcl["PaPdTbPc"],
+                                              wcl["TaTcPbTd"], wcl["TaTdPbTc"], wcl["PaTcTbTd"], wcl["PaTdTbTc"],
+                                              wcl["PaTcPbTd"], wcl["PaTdPbTc"], wcl["PaTcTbPd"], wcl["PaPdTbTc"],
+                                              wcl["PaTcPbPd"], wcl["PaPdPbTc"], wcl["PaPcPbTd"], wcl["PaTdPbPc"],
+                                              doPlanck, l_exact, l_band, l_toep, coupling.T)
+
 
         indexlist = np.arange(32)
         for name, index in zip(win_list, indexlist):
-            coupling_dict[name] = coupling[index] + coupling[index].T - np.diag(np.diag(coupling[index]))
+            if l_toep < lmax:
+                coupling[index] = so_mcm.format_toepliz(coupling[index], l_toep, lmax)
+            mcm_fortran.fill_upper(coupling[index].T)
+            coupling_dict[name] = coupling[index]
 
     if save_file is not None:
         np.save("%s.npy"%save_file, coupling)
